#include <stdarg.h>
#include <stddef.h>
#include <inttypes.h>
#include <string.h>
#include <stdio.h>

size_t
VSLwv(void *buf, const char *fmt, va_list ap)
{
	void *p = buf;
	size_t l = 0;
	const char *s;

	for (int i = 0; fmt[i]; i++) {
		switch (fmt[i]) {
		case '%':
		case '.':
			continue;
		case 's':
			s = va_arg(ap, const char *);
			if (l == 0)
				l = strlen(s);
			*((unsigned short *)p) = (unsigned short)l;
			p += sizeof(unsigned short);
			memcpy(p, s, l);
			p += l;
			l = 0;
			break;
		case 'd':
		case 'u':
		case 'x':
			*((int *)p) = va_arg(ap, int);
			p += sizeof(int);
			break;
		case 'f':
			*((double *)p) = va_arg(ap, double);
			p += sizeof(double);
			break;
		case '*':
			l = (size_t)va_arg(ap, int);
			p += sizeof(int);
			break;
		case 'j':
			*((intmax_t *)p) = va_arg(ap, intmax_t);
			p += sizeof(intmax_t);
			i++;
			break;
		case 'z':
			*((size_t *)p) = va_arg(ap, size_t);
			p += sizeof(size_t);
			i++;
			break;
		case 'h':
			*((short *)p) = (short) va_arg(ap, int);
			p += sizeof(short);
			i++;
			break;
		}
	}
	return (p - buf);
}

size_t
VSLw(void *buf, const char *fmt, ...)
{
	va_list ap;
	size_t sz;

	va_start(ap, fmt);
	sz = VSLwv(buf, fmt, ap);
	va_end(ap);
	return (sz);
}

void
hex(char *buf, size_t len)
{
	for (int i = 0; i < len; i++)
		printf("%02x", (int) buf[i]);
	printf("\n");
}

int
main(int argc, char *argv[])
{
	char buf[255];
	size_t len;
	unsigned short l;
#if 0
	len = VSLw(buf, "%f%f%f%s", 1554975889.464424, 0.000013, 0.000002,
		   "Request");
	printf("len=%zd\n", len);
	hex(buf, len);
	printf("%.*s: %f %f %f\n\n", *((unsigned short *)&buf[24]), &buf[26],
	       *((double *)&buf[0]), *((double *)&buf[8]),
	       *((double *)&buf[16]));

	len = VSLw(buf, "%s%s", "Host", "varnish-cache.org");
	printf("len=%zd\n", len);
	hex(buf, len);
	l = *((unsigned short *)&buf[0]);
	printf("%.*s: %.*s\n\n", l, &buf[2], *((unsigned short *)&buf[2+l]),
	       &buf[4+l]);

	len = VSLw(buf, "%ju%ju%ju%ju%ju%ju", 110, 0, 110, 210, 224, 434);
	printf("len=%zd\n", len);
	hex(buf, len);
	l = *((unsigned short *)&buf[0]);
	printf("%ju %ju %ju %ju %ju %ju\n\n", *((uintmax_t *)&buf[0]),
	       *((uintmax_t *)&buf[1 * sizeof(uintmax_t)]),
	       *((uintmax_t *)&buf[2 * sizeof(uintmax_t)]),
	       *((uintmax_t *)&buf[3 * sizeof(uintmax_t)]),
	       *((uintmax_t *)&buf[4 * sizeof(uintmax_t)]),
	       *((uintmax_t *)&buf[5 * sizeof(uintmax_t)]));

	len = VSLw(buf, "%hu", 200);
	printf("len=%zd\n", len);
	hex(buf, len);
	printf("%hu\n\n", *((unsigned short *)&buf[0]));
#endif
	for (int i = 0; i < 100000000; i++) {
		VSLw(buf, "%f%f%f%s", 1554975889.464424, 0.000013, 0.000002,
		     "Request");
		VSLw(buf, "%s%s", "Host", "varnish-cache.org");
		VSLw(buf, "%ju%ju%ju%ju%ju%ju", 110, 0, 110, 210, 224, 434);
		VSLw(buf, "%hu", 200);

		sprintf(buf, "%f%f%f%s", 1554975889.464424, 0.000013, 0.000002,
			"Request");
		sprintf(buf, "%s%s", "Host", "varnish-cache.org");
		sprintf(buf, "%ju%ju%ju%ju%ju%ju", 110, 0, 110, 210, 224, 434);
		sprintf(buf, "%hu", 200);
	}
}
